float cirrusCloudFBM(vec2 position) {
    position *= 0.00002;

    float noise      = 0.0;
    float local      = 0.0;
    float amplitude  = 1.0;
    float weight     = 0.0;
    mat2  rotMat     = mat2(3.2, 1.2, -3.2, 1.2);

    vec2  movement   = frameTimeCounter * windDirection * 0.001 * Cirrus_Speed;

    vec3 distortion  = noiseSmooth(position + movement).xyz;

    { // Standard noise pattern
        position.x += distortion.x * 0.01;
        noise      += noiseSmooth(position + movement).b * amplitude;
        weight     += amplitude;

        position   *= 2.5;
        amplitude  *= 0.8;
        position.y += distortion.y * 0.015;
        noise      += noiseSmooth(position + movement).b * amplitude * noise;
        weight     += amplitude;

        position   *= 2.5;
        amplitude  *= 0.2;
        position   += distortion.xz * 0.005;
        noise      += noiseSmooth(position + movement).b * amplitude * noise;
        weight     += amplitude;

        position   *= 4.5;
        amplitude  *= 0.15;
        position.x += noise * 0.1;
        noise      += noiseSmooth(position + movement).b * amplitude;
        weight     += amplitude;

        position   *= 5.5;
        amplitude  *= 0.3;
        noise      += noiseSmooth(position + movement).b * amplitude;
        weight     += amplitude;
    }

    noise /= weight;

    return pow(noise, 10.0);
}

void calculateCirrusClouds(inout vec3 color, in vec3 worldSpace, in vec3 worldSpaceNormalized, in float visibility) {
    #if !defined Cirrus_Clouds
        return;
    #endif

    float horizon = smoothstep(0.05, 0.1, worldSpaceNormalized.y);

    if (horizon <= 0.0 || visibility > 0.5) return;

    // Precalculate integral variables
	const float a = -mieCoefficient.x / log(2.0);
	const float b = -1.0 / mieCoefficient.x;
	const float c =  1.0 / mieCoefficient.x;

    const float thickness  = 50.0;
    const float extinction = 1.0;

    vec3  lightCol = lightColors[0] + lightColors[1];


    vec3  ray = worldSpace * (Cirrus_Clouds_Altitude - cameraPosition.y) / worldSpace.y + cameraPosition;

    float VoL = dot(worldSpaceNormalized, lightVec[1]);

    float opticalDepth    = cirrusCloudFBM(ray.xz) * thickness;
    float lightVisibility = exp2(-mieCoefficient.x * cirrusCloudFBM(ray.xz + lightVec[1].xz * 1.0) * thickness * extinction);
    float phase           = hgPhase(VoL, 0.9) * 0.1;

    vec3 lighting = lightVisibility * lightCol * phase * 0.1;

    vec4  clouds       = vec4(
        lighting * (exp2(a * opticalDepth) * b + c),
        exp2(-mieCoefficient.x * opticalDepth * extinction)
    );

    clouds = mix(vec4(0.0, 0.0, 0.0, 1.0),
             clouds,
             horizon);

    color = color * clouds.a + clouds.rgb;
}