const float maxCloudsAltitude      = Volumetric_Clouds_Altitude + Volumetric_Clouds_Height;

float calculateCloudNoise(vec3 position, bool doDetailNoise) {
    position *= 0.0025;
    
    float coverage = mix(0.5 * Volumetric_Cloud_Coverage, 0.3, rainStrength);

    float noise      = 0.0;
    float amplitutde = 1.0;
    float weight     = 0.0;

    vec3  movement = vec3(windDirection.x, 0.0, windDirection.y) * frameTimeCounter * 0.05;

    { // Large noise calculation
        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
        position   *= 3.5;
        amplitutde *= 0.8;

        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
        position   *= 2.5;
        amplitutde *= 0.35;
    
        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
        position   *= 2.3;
        amplitutde *= 0.65;

        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
        position   *= 2.0;
        amplitutde *= 0.7;
        
        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
        position   *= 4.0;
        amplitutde *= 0.5;
    }

    if (doDetailNoise) { // Small noise calculation

        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
        position   *= 31.5;
        amplitutde *= 0.1;

        noise      += Get3DNoise(position + movement) * amplitutde;
        weight     += amplitutde;
    }
/*
    float t = frameTimeCounter;
    vec3  p = position * 3.0;

    t *= 0.0095;

	p.x *= 0.3f;
	p.x -= t * 0.01f;

	vec3 p1 = p * vec3(1.0f, 0.5f, 1.0f)  + vec3(0.0f, t * 0.01f, 0.0f);

	noise  = 	Get3DNoise(p * vec3(1.0f, 0.5f, 1.0f) + vec3(0.0f, t * 0.01f, 0.0f)) * 1.3;
    p *= 2.0f;	p.x -= t * 0.557f;
    vec3 p2 = p;

    noise += (2.0f - abs(Get3DNoise(p) * 2.0f - 0.0f)) * (0.35f);
    p *= 3.0f;
    p.xz -= t * 0.905f;
    p.x *= 2.0f;
    vec3 p3 = p;
    float largeNoise = noise;

    noise += (3.0f - abs(Get3DNoise(p) * 3.0f - 0.0f)) * (0.085f);
    p *= 3.0f;
    p.xz -= t * 3.905f;
    vec3 p4 = p;

    noise += (3.0f - abs(Get3DNoise(p) * 3.0f - 0.0f)) * (0.035f);
    p *= 3.0f;
    p.xz -= t * 3.905f;

    noise += ((Get3DNoise(p))) * (0.04f);
    p *= 3.0f;
    noise /= 2.375f;
    */
    return max0(noise / weight - coverage);
}

float getCloudVolume(vec3 ray, float distance, bool doDetailNoise) {
	
	if (ray.y < Volumetric_Clouds_Altitude || ray.y > maxCloudsAltitude) return 0.0;

    const float density = 0.6 * Volumetric_Cloud_Density;

    float altitudeLayer = ray.y - (Volumetric_Clouds_Altitude + Volumetric_Clouds_Height * 0.5);
    float gradientLayer = pow(min1(abs(altitudeLayer) / Volumetric_Clouds_Height), 3.5);
    float opticalDepth  = max0(calculateCloudNoise(ray, doDetailNoise) - distance - gradientLayer) * (1.0 - gradientLayer);

    return opticalDepth * density;
}

float calculateCloudShadows(vec3 worldSpace) {
    #if !defined Cloud_Shadows || !defined Volumetric_Clouds
        return mix(1.0, 0.0, rainStrength);
    #endif

    float horizon = smoothstep(0.2, 0.5, lightVec[1].y);

    const float midCloudAltitude = Volumetric_Clouds_Altitude + Volumetric_Clouds_Height * 0.5;

    vec3 ray = worldSpace + lightVec[1] * ((midCloudAltitude - worldSpace.y) / lightVec[1].y);

    float opticalDepth = mix(getCloudVolume(ray, 0.0, false) * horizon, 1.0, rainStrength * (1.0 - horizon));

    return exp2(-(Volumetric_Clouds_Height) * opticalDepth);
}

float beerLambert(float sampleDensity, float precipitation) {
	return exp(-sampleDensity * precipitation);
}

float powder(float sampleDensity, float lightDotEye) {
	float powd = 1.0f - exp(-sampleDensity * 2.0f);
	return mix(
		1.0f,
		powd,
		clamp01((-lightDotEye * 0.5) + 0.5) // [-1,1]->[0,1]
	);
}

float calculateSunlightVisibility(vec3 ray) {
     int   samples  = VC_Direct_Light_Quality;
     float stepSize = Volumetric_Clouds_Height / samples;
    
	vec3 rayDirection = lightVec[1] * stepSize;
	ray              += rayDirection * 0.1;

	float opticalDepth = 0.0;
	for (float i = 0.0; i < samples; i++, ray += rayDirection)
		opticalDepth += getCloudVolume(ray, 0.0, true);

	return exp2(-opticalDepth * stepSize * RCPLOG2);
}

float calculateSkylightVisibility(vec3 ray) {
     int   samples  = 1;
     float stepSize = Volumetric_Clouds_Height / samples / 10;
    
	vec3 rayDirection = vec3(0.0, -1.0, 0.0) * stepSize;
	ray              += rayDirection * 1.15;

	float opticalDepth = 0.0;
	for (float i = 0.0; i < samples; i++, ray += rayDirection)
		opticalDepth += getCloudVolume(ray, 0.0, true);

	return exp2(-opticalDepth * stepSize * RCPLOG2 * 0.5);
}

/*
float calculateSkylightVisibility(vec3 ray, float startingOpticalDepth) {
    const int   samples  = 1;
    const float stepSize = Volumetric_Clouds_Height / (samples);

	vec3 rayDirection = vec3(0.0, 0, -1.0) * stepSize;
	ray              += rayDirection * 1.15;

	float opticalDepth = -0.0 * startingOpticalDepth;
	for (float i = 0.0; i < samples; i++, ray += rayDirection) {
		opticalDepth -= getCloudVolume(ray);
	}
	return exp(0.02 * stepSize * opticalDepth) - 0.06;
}
*/


vec4 calculateVolumetricClouds(gbuffer info, material mat, scene spaces, vec2 pattern) {
    #ifndef Volumetric_Clouds
        return vec4(0.0, 0.0, 0.0, 1.0);
    #endif

    if (info.depthtex.x < 1.0) return vec4(0.0, 0.0, 0.0, 1.0);

    const float heightFraction = Volumetric_Clouds_Height / Volumetric_Clouds_Quality;

    vec3  planeBottom = spaces.world[1] * (Volumetric_Clouds_Altitude - cameraPosition.y) / spaces.world[1].y;
          planeBottom = spaces.world[2] * min(length(planeBottom), Volumetric_Cloud_Distance);
    vec3  planeTop    = spaces.world[1] * (maxCloudsAltitude - cameraPosition.y) / spaces.world[1].y;
          planeTop    = spaces.world[2] * min(length(planeTop), Volumetric_Cloud_Distance);

    vec3 rayStart  = vec3(0.0);
    vec3 rayEnd    = vec3(0.0);

    if (cameraPosition.y < Volumetric_Clouds_Altitude) {
        rayStart = planeBottom;
        rayEnd   = planeTop;
    } else if (cameraPosition.y > Volumetric_Clouds_Altitude && cameraPosition.y < maxCloudsAltitude) {
        rayStart = vec3(0.0);
        rayEnd   = spaces.world[2].y < 0.0 ? planeBottom : planeTop;
    } else if (cameraPosition.y > maxCloudsAltitude) {
        rayStart = planeTop;
        rayEnd   = planeBottom;
    }

    if (length(rayStart) >= Volumetric_Cloud_Distance) return vec4(0.0, 0.0, 0.0, 1.0);

    vec3  rayIncrement = (rayEnd - rayStart) / Volumetric_Clouds_Quality;
    vec3  rayPosition  = rayIncrement * pattern.x + rayStart;
    float stepSize     = length(rayIncrement);
    
    vec3  lightCol = lightColors[0] + lightColors[1];
    float VoL      = dot(spaces.world[2], lightVec[1]);
    float phase    = hgPhase(VoL, 0.22);

    vec4 clouds = vec4(0.0, 0.0, 0.0, 1.0);

    for (int i = 0; i < Volumetric_Clouds_Quality; i++, rayPosition += rayIncrement) {
        vec3  rayWorldPos    = rayPosition + cameraPosition;
        float opticalDensity = getCloudVolume(rayWorldPos, 0.0, true);

        if (opticalDensity <= 0.0) continue;

        vec3 lighting  = vec3(0.0);
             lighting += calculateSkylightVisibility(rayWorldPos) * vec3((0.0, 0.4, 1.0) * 5.0) * 0.5;
             lighting += calculateSunlightVisibility(rayWorldPos) * lightCol * phase * 0.7;
             lighting += exp(-opticalDensity * heightFraction * 1.11) * RCPPI * lightColors[3] * 0.3;
             //lighting *= mix(1.0, 1.0 - exp(-opticalDensity * 2.0), -VoL * 0.5 + 0.5);

        clouds.rgb += opticalDensity * heightFraction * lighting * clouds.a;
        clouds.a   *= exp(-opticalDensity * 0.5 * stepSize * 1.11 * RCPLOG2);
    }

    return clouds;
}