float volumetricFogFBM(vec3 position) {
    #ifndef Volumetric_Fog
    return 0.0;
    #endif

    position = position * 0.025;

    float coverage = 0.0;

    float noise = 0.0;
    float a = 1.0;
    float weight = a;

    mat2  rotMat   = mat2(2.2, 1.2, -3.2, 2.2);
    float movement = frameTimeCounter * 0.113 * Volumetric_Fog_Speed;

    for (int i = 0; i < 3; i ++) {
        noise    += Get3DNoise(position + movement) * a;
        position *= 4.5;
        a        *= 0.32;
        weight   += a;
    }

    noise /= weight;
    noise  = pow(max0(noise - coverage) / (1.0 - coverage), 15.0);

    return noise;
}

float calculateFogOpticalDensity(vec3 position) {
    float opticalDensity  = exp(-(position.y + Volumetric_Clouds_Altitude) / Volumetric_Clouds_Altitude) * 1.0e3 * pow2(VL_Thickness * 2);
          opticalDensity += exp(-max0(position.y - 80.0) / 10.0) * volumetricFogFBM(position) * 1.0e1;
    return opticalDensity;
}

vec4 calculateVolumetricLight(gbuffer info, material mat, scene spaces, vec2 pattern) {
    #ifndef Volumetric_Light
        return vec4(0.0, 0.0, 0.0, 1.0);
    #endif

    // Setup scattering / extinction integral
	const float a = -mieExtinction.x * RCPLOG2;
	const float b = -1.0 / mieExtinction.x;
	const float c =  1.0 / mieExtinction.x;

    float vlRenderDistance = mix(Volumetric_Light_Distance, Volumetric_Light_Distance_Rain, rainStrength);

    vec3  planeBottom = spaces.world[0] * (-cameraPosition.y) / spaces.world[0].y;
          planeBottom = spaces.world[2] * min(length(planeBottom), vlRenderDistance);
    vec3  planeTop    = spaces.world[0] * (Volumetric_Clouds_Altitude - cameraPosition.y) / spaces.world[0].y;
          planeTop    = spaces.world[2] * min(length(planeTop), vlRenderDistance);

    vec3 rayStart  = vec3(0.0);
    vec3 rayEnd    = vec3(0.0);

    if (cameraPosition.y < -Volumetric_Clouds_Altitude) {
        rayStart = planeBottom;
        rayEnd   = planeTop;
    } else if (cameraPosition.y > -Volumetric_Clouds_Altitude && cameraPosition.y < Volumetric_Clouds_Altitude) {
        rayStart = vec3(0.0);
        rayEnd   = spaces.world[2].y < 0.0 ? planeBottom : planeTop;
    } else if (cameraPosition.y > Volumetric_Clouds_Altitude) {
        if (spaces.world[2].y > 0.0) return vec4(0.0, 0.0, 0.0, 1.0);
        rayStart = planeTop;
        rayEnd   = planeBottom;
    }

    rayEnd = info.depthtex.x < 1.0 ? spaces.world[0] : rayEnd;

    vec3  rayIncrement = (rayEnd - rayStart) / Volumetric_Light_Quality;
    vec3  rayPosition  = rayIncrement * pattern.x + rayStart;
    float stepSize     = length(rayIncrement);
    
    float VoL     = dot(spaces.world[2], lightVec[1]);
          VoL     = min1(( VoL + sinSunAngle) / (1.0 + sinSunAngle));
    float phase   = hgPhase(VoL, 0.6);
    float indoors = pow2(eyeBrightnessSmooth.y / 240.0);

    vec4 volumetricLight = vec4(0.0, 0.0, 0.0, 1.0);

    for (int i = 0; i < Volumetric_Light_Quality; i++, rayPosition += rayIncrement) {
        float opticalDensity = calculateFogOpticalDensity(rayPosition + cameraPosition) * stepSize;
        float cloudShadow    = calculateCloudShadows(rayPosition + cameraPosition);
        float landShadow     = calculateShadow(shadowtex0, distortShadowSpace(calculateShadowSpace(rayPosition)));

        vec3 lightingSun     = landShadow * cloudShadow * lightColors[2];
        vec3 lightingAmbient = indoors * RCPPI * lightColors[3];

        float scatteringScale = scatterCoefficients[1].x * (exp2(a * opticalDensity) * b + c);
        
        volumetricLight.rgb += (scatteringScale * phase) * lightingSun * volumetricLight.a * vec3( 1.5, 1.1, 1.0);
        volumetricLight.rgb += (scatteringScale * (0.25)) * lightingAmbient * volumetricLight.a;
        volumetricLight.a   *= exp(-extinctionCoefficients[1].x * opticalDensity * RCPLOG2);
    }

    return volumetricLight;
}