/*                         */
/* START DEVELOPER OPTIONS */
/*                         */

/* Setup Programs */

// Gbuffer program types
#define G_DISCARD 0
#define G_SOLID   1
#define G_FORWARD 2
// PROG_SHADOW only
#define G_SHADOW  3
#define G_VOXEL   4

// Set gbuffers to use program
#define PROG_BASIC       G_SOLID
#define PROG_ENTITIES    G_SOLID
#define PROG_HAND        G_SOLID
#define PROG_SKYBASIC    G_DISCARD
#define PROG_SKYTEXTURED G_DISCARD
#define PROG_TERRAIN     G_SOLID
#define PROG_TEXTURED    G_FORWARD
#define PROG_WATER       G_FORWARD
#define PROG_WEATHER     G_FORWARD
#define PROG_SHADOW      G_SHADOW

/* Setup Buffers */
/*
const int colortex0Format = RGBA16F;
const int colortex1Format = RGBA16;
const int colortex2Format = RGB16;
const int colortex3Format = RGBA8;
const int colortex4Format = RGBA16;
const int colortex5Format = RGBA16;
const int colortex6Format = RGBA16F;
const int colortex7Format = RGBA8;

const bool colortex0Clear = true;
const bool colortex1Clear = true;
const bool colortex2Clear = true;
const bool colortex3Clear = true;
const bool colortex4Clear = false;
const bool colortex5Clear = false;
const bool colortex6Clear = false;
const bool colortex7Clear = false;
*/
/* Initialize Shader */

#define Wind_Direction_Degrees 45.0

const int   noiseTextureResolution  = 64;
const float noiseResInverse         = 1.0 / noiseTextureResolution;

const float sunPathRotation	        = -40;
const int   shadowMapResolution     = 2048;  //[512 1024 2048 4096 6046]
const float shadowDistance          = 120.0;

const float shadowMapResInverse     = 1.0 / shadowMapResolution;

const float sunAngularRadius  = radians(0.75);
const float sinSunAngle       = sin(sunAngularRadius);
const float cosSunAngle       = cos(sunAngularRadius);

const float windDirectionRad  = radians(Wind_Direction_Degrees);
const vec2  windDirection     = vec2(sin(windDirectionRad), cos(windDirectionRad));

/* Setup option toggles */

// General
#define Off           0
#define On            1

// Diffuse
#define Lambertian    0
#define Hammon_GGX    1

// Emission
#define Luma_Based    0
#define Resource_Pack 1

/*                         */
/*  END DEVELOPER OPTIONS  */
/*                         */

/*  ACES Settings  */

//#define ACES_Tonemap

#define SAT_MOD 0.0 // [-1.0 -0.9 -0.8 -0.7 -0.6 -0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define VIB_MOD 0.0 // [-1.0 -0.9 -0.8 -0.7 -0.6 -0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define CONT_MOD 0.0 // [-0.4 -0.3 -0.2 -0.1 -0.09 -0.08 -0.07 -0.06 -0.05 -0.04 -0.03 -0.02 -0.01 0.0 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.1 0.2 0.3 0.4]
#define CONT_MIDPOINT 0.1 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define GAIN_MOD -0.3 // [-1.0 -0.9 -0.8 -0.7 -0.6 -0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define LIFT_MOD 0.1 // [-1.0 -0.9 -0.8 -0.7 -0.6 -0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define WHITE_BALANCE 6500 //[1000 1500 2000 2500 3000 3500 4000 4500 5000 5500 6000 6500 7000 7500 8000 8500 9000 9500 10000]

#define Film_Slope 1.1 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Film_Toe 0.3// [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Film_Shoulder 0.3 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Black_Clip 0.002 // [0.000 0.001 0.002 0.003 0.004 0.005 0.006 0.007 0.008 0.009 0.010]
#define White_Clip 1.0 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Blue_Correction 0.0 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Gamut_Expansion 0.0 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Exposure 0.8 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0 5.0]

#define in_Match 0.4  // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Out_Match 0.3 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

/*     Colors     */

#define Filmic_Tonemap

//#define Color_Grading

//Atmosphere
#define Atmosphere_Red 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Atmosphere_Green 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Atmosphere_Blue 1.1 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Atmosphere_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//

//Emitters
#define Emitter_Red 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Emitter_Green 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Emitter_Blue 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

#define Emitter_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//

//Sun
#define Sun_Red 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Sun_Green 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Sun_Blue 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Sunlight_Amount 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 3.0 4.0]
//

//Moon
#define Moon_Red 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Moon_Green 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Moon_Blue 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Moonlight_Amount 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0]
//

//Nebula Star
#define Star_Red 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Star_Green 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Star_Blue 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//

//Cloud Lighting
#define Cloud_Direct_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Cloud_Indirect_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//

//VL
#define VL_Red 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define VL_Green 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define VL_Blue 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//

//Stars
#define Stars_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//

const vec3  sunTint    = vec3(50.0, 50.0, 50.0);
const vec3  moonTint   = vec3(0.9, 0.9, 0.8) * 0.35;
const vec3  torchColor = vec3(0.7 * Emitter_Red, 0.24 * Emitter_Green, 0.07 * Emitter_Blue) * 60.0 * Emitter_Brightness;

/*      Other      */

#define C0_Scale 0.75

#define Emissive_Mapping Luma_Based //[Luma_Based Resource_Pack]
#define Emissive_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0 5.0]
//#define Gray_Scale_Specular_Support

#define Accumulator_Weight 0.98 //[0.0 0.5 0.8 0.9 0.95 0.98 0.99]

#define Shadow_Bias 0.9
#define Shadow_Z_Stretch 1.5

#define horizonAltitude 500.0

#define Ambient_Occlusion
#define AO_Quality 2 // [2 3 4 5 6 8 10 15 20 25 30 35 40 50 100]
#define AO_Strength 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//#define Stochastic_AO

#define DepthOfField
#define DepthOfFieldQuality 16   // [8 16 24 32 40 48 56 64 72 80 88 96 104 112 120 128]
#define DOF_Bias            8.0  // [1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0]
#define DOF_Anamorphic      1.0  // [1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define DOF_Aperture        15   // [10 15 20 25 30 35 40 45 50 55 60 65 70 75]
#define Focal_Length        35.0 // [0.0 5.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 45.0 50.0]
#define Aperture_Blades     4    // [2 3 4 5 6 7 8 9 10]
#define Aperture_Rotation   60.0
//#define Tilt_Shift
//#define Distance_Blur

#define Motion_Blur

//#define Diffraction_Spikes
#define Diffraction_Spikes_Quality 4

#define Lens_Flare

//#define Helmet_Distortion

#define PCSS // Percentage Closer Soft Shadows
//#define HRTS // Raytraced contact shadows, a bit buggy

#define HRTS_Samples 4 //[3 4 5 6 7 8 9 10 15 20]
#define HRTS_Range 6.0 //[2.0 4.0 6.0 8.0 10.0 20.0 100.0]

//#define RSM
#define RSM_Rays 2 //[2 3 4 5 6 7 8 9 10]
#define RSM_Radius 50.0 //[10.0 20.0 30.0 40.0 50.0 60.0 70.0 80.0 90.0 100.0 150.0 200.0]

#define RSM_Brightness 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0]

#define Specular_Rays 1 //[0 1 2 3 4 5 6 7 8 9 10]

#define LUT
//#define Show_LUT

#define Selected_LUT 3 //[0 1 2 3 4 5 6 7]

#define Bloom_Brightness 1.7 // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0]

//#define Dirty_Lens

#define Parallax_Occlusion
#define Parallax_Depth 0.15 // [0.1 0.15 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0]

//#define Cirrus_Clouds
#define Cirrus_Clouds_Altitude 600.0 //[100 150 200 250 300 350 400 450 500 550 600 650 700 750 800 850 900 950 1000]
#define Cirrus_Speed 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0]

#define Volumetric_Clouds
#define Volumetric_Clouds_Altitude        170.0   // [0.0 10.0 20.0 30.0 40.0 50.0 60.0 70.0 80.0 90.0 100.0 110.0 120.0 130.0 140.0 150.0 160.0 170.0 180.0 190.0 200.0 210.0 220.0 230.0 240.0 250.0 260.0 270.0 280.0 290.0 300.0 310.0 320.0 330.0 340.0 350.0 360.0 370.0 380.0 390.0 400.0 450.0 500.0 600.0 700.0 800.0 900.0 1000.0 2000.0 3000.0]
#define Volumetric_Cloud_Density          1.0     // [0.1 0.15 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0]
#define Volumetric_Clouds_Height          110.0   // [70.0 80.0 90.0 100.0 110.0 120.0 130.0 140 150 200 250 500]
#define Volumetric_Clouds_Quality         10       // [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20]
#define Volumetric_Clouds_Lighting_Amount 1.0     // [1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0 10.0] Raise this as you lower direct light steps, results may vary.
#define Volumetric_Cloud_Distance         10000.0 // [1000.0 1500.0 2000.0 2500.0 3000.0 3500.0 4000.0 4500.0 5000.0 10000.0 15000.0 20000.0 30000.0]
#define Volumetric_Cloud_Coverage         1.0    // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define VC_Direct_Light_Quality           3       // [1 2 3 4 5 6 7 8 9 10]
#define Cloud_Shadows

#define Clouds_Multi

#define Volumetric_Light
#define Volumetric_Light_Quality       3    // [1 2 3 4 5 6 7 8 9 10 12 15 20]
#define Volumetric_Light_Distance      1000 // [50 100 500 1000 1500 2000 3000 4000 5000 6000 7000 8000 9000 10000]
#define Volumetric_Light_Distance_Rain 500  // [50 100 500 1000 1500 2000 3000 4000 5000 6000 7000 8000 9000 10000]
#define VL_Thickness 1.0 // [0.1 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0]

#define Volumetric_Fog // Requires Volumetric Light to be enabled
#define Volumetric_Fog_Thickness 1.5 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 5.0 10.0]
#define Volumetric_Fog_Speed 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

#define Skylight
#define Skylight_Brightness 1.2 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0]

#define Atmosphere_Steps 10 //[5 10 11 12 13 14 15 16 17 18 19 20 40 50 100]

#define Water_Speed 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 3.0 4.0]
#define Water_Parallax_Samples 0 // [1 2 3 4 5 6 7 8 9 10 20 40 50]