mat4x3 calculateLightColors(vec3 sunVec) {
    // Calculates light colors, [0] sunlight, [1] moonlight, [2] combined light, [3] ambient light
    const int steps = 15;
    const vec3 earthPosition = vec3(0.0, earthRadius + horizonAltitude, 0.0);

    vec3   lightIntersect = vec3(
        raySphereDepth( sunVec, -earthPosition, atmosphereRadius),
        raySphereDepth(-sunVec, -earthPosition, atmosphereRadius),
        atmosphereHeight
    );

    vec3 stepSize = lightIntersect / steps;

    mat3 rayDirections = mat3(
         sunVec * lightIntersect.x,
        -sunVec * lightIntersect.y,
        vec3(0.0, 1.0, 0.0) * lightIntersect.z
    );

    mat3 increments = mat3(
        rayDirections[0] / steps,
        rayDirections[1] / steps,
        rayDirections[2] / steps
    );

    mat3 rays = mat3(
        earthPosition,
        earthPosition,
        earthPosition
    );

    mat3 transmission = mat3(
        vec3(1.0),
        vec3(1.0),
        vec3(1.0)
    );
    
    vec3 scatter = vec3(0.0);

    for (int i = 0; i < steps; i++, rays += increments) {
        mat3 opticalDepth = mat3(
            calculateOpticalDensities(length(rays[0]) - earthRadius) * stepSize.x,
            calculateOpticalDensities(length(rays[1]) - earthRadius) * stepSize.y,
            calculateOpticalDensities(length(rays[2]) - earthRadius) * stepSize.z
        );

        scatter += scatterCoefficients * opticalDepth[2].xy * transmission[2];
        transmission[0] *= exp(extinctionCoefficients * -opticalDepth[0]);
        transmission[1] *= exp(extinctionCoefficients * -opticalDepth[1]);
        transmission[2] *= exp(extinctionCoefficients * -opticalDepth[2]);
    }

    mat4x3 matrix;
           matrix[0] = transmission[0] * sunTint * Sunlight_Amount * vec3(Sun_Red, Sun_Green, Sun_Blue) * 4;
           matrix[1] = transmission[1] * moonTint * Moonlight_Amount * vec3(Moon_Red, Moon_Green, Moon_Blue);
           matrix[2] = matrix[0] * smoothstep(0.0, 0.005, sunVec.y) + matrix[1] * smoothstep(0.0, 0.005, -sunVec.y);
           matrix[3] = scatter * mix(matrix[0] + matrix[1], vec3(length(matrix[0] + matrix[1])), 0.5) * 3.0 * vec3(pow2(Atmosphere_Red), pow2(Atmosphere_Green), pow2(Atmosphere_Blue));
           
    return matrix;
}