/* DRAWBUFFERS:32 */

layout (location = 0) out vec4 colortex3Write;
layout (location = 1) out vec4 colortex2Write;


in vec4 color;
in vec3 worldSpace;
in vec3 viewSpace;
in vec3 tangentViewSpace;
in vec2 texcoord;
in vec2 vertexLightmaps;

flat in mat3 tbnMatrixWorld;
flat in mat3 tbnMatrixView;
flat in int materialIDs;

uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D noisetex;
uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;
uniform float viewWidth, viewHeight;
uniform float frameTimeCounter;
uniform int frameCounter;

const vec2 pixel = 1.0 / vec2(viewWidth, viewHeight);


#include "/lib/Utility/Temporal.glsl"

#include "/lib/Fragment/Sampling_Patterns.glsl"
#include "/lib/Fragment/Water_Waves.glsl"


vec4 getAlbedo(vec2 coord) {
    if (materialIDs == 1) return vec4(1.0, 1.0, 1.0, 0.5);

    vec4 albedo     = texture2D(texture, coord);
         albedo.rgb = toLinear(albedo.rgb * color.rgb);
    
    return albedo;
}

vec4 getTangentNormals(vec2 coord) {
    vec4 normal = texture2D(normals,  coord) * 2.0 - 1.0;
    calculateProceduralNormals(normal.xyz);
    normal.xyz = clampNormal(normal.xyz, -normalize(tangentViewSpace));
    return normal;
}

vec3 calculateSpecularMaps(vec2 coord) {
    vec3 specular = texture2D(specular, coord).rgb;
    #ifdef Gray_Scale_Specular_Support
        specular = vec3(specular.r, 0.0, 0.0);
    #endif
    return specular;
}

vec2 calculateLightmaps(vec3 normalTex, vec3 normal, vec2 pattern) {
    vec2 lightmaps   = vertexLightmaps;
         lightmaps.x = 16.0 - min(15.0, (lightmaps.x - 0.5 / 16.0) * 16.0 * 16.0 / 15.0);
	     lightmaps.x = pow2(clamp01(1.0 - pow4(lightmaps.x / 16.0))) / (1.0 + pow2(lightmaps.x));
         lightmaps.x = mix(lightmaps.x, 0.2, float(materialIDs == 3));

    return sqrt(lightmaps) + pattern.x / vec2(31,63);
}


void main() {
    vec2 coord     = texcoord;

    vec2 pattern   = calculateJitteredSamplingDistributors();

    vec4 albedo    = getAlbedo(coord);
    vec4 normalTex = getTangentNormals(coord);
    vec3 specular  = calculateSpecularMaps(coord);
    vec3 normal    = normalize((normalTex.xyz) * tbnMatrixWorld);
    vec2 lightmaps = calculateLightmaps(normalTex.xyz, normal, pattern);


    colortex3Write = albedo;
    colortex2Write = vec4(
        encodeNormal3x16(normal),
        encode3x16(vec3(lightmaps, encodeMaterialIDs(materialIDs))),
        encode3x16(specular),
        1.0
    );
}