/* DRAWBUFFERS:01 */

layout (location = 0) out vec4 outAlbedo;
layout (location = 1) out vec4 outPassthrough;


in vec3 color;
in vec3 worldSpace;
in vec2 texcoord;

flat in vec3 normal;
flat in int materialIDs;

uniform sampler2D texture;
uniform sampler2D specular;
uniform sampler2D noisetex;
uniform mat4 gbufferModelViewInverse;
uniform vec3 shadowLightPosition;
uniform vec3 cameraPosition;
uniform float frameTimeCounter;


#include "/lib/Fragment/Noise.glsl"
#include "/lib/Fragment/Mapping.glsl"
#include "/lib/Fragment/Water_Waves.glsl"


vec4 getAlbedo(vec2 coord) {
    if (materialIDs == 1) return vec4(0.5);
    vec4 albedo     = texture2D(texture, coord);
         albedo.rgb = albedo.rgb * color;
    return albedo;
}

vec3 calculateSpecularMaps(vec2 coord) {
    vec3 specular = texture2D(specular, coord).rgb;
    #ifdef Gray_Scale_Specular_Support
         specular = vec3(specular.r, 0.0, 0.0);
    #endif
    return specular;
}


void main() {
    vec4 albedo   = getAlbedo(texcoord);
    vec3 specular = calculateSpecularMaps(texcoord);
    vec3 normals  = materialIDs == 1 ? calculateProceduralNormalsFlat(worldSpace) : normal;

    if (materialIDs == 1) {
        const float depth = 1.0;
        vec3 lightVec = mat3(gbufferModelViewInverse) * normalize(shadowLightPosition);

		vec3  flatRefractVector = refract(-lightVec, vec3(0.0, 1.0, 0.0), 0.75);
		float surfDistUp        = -depth * abs(lightVec.y);

		vec3 rc = flatRefractVector * (surfDistUp / flatRefractVector.y);

        vec3 samplePos     = worldSpace - rc;

        vec3 refractVector = refract(-lightVec, -calculateProceduralNormalsFlat(samplePos), 0.75);
             samplePos     = refractVector * (surfDistUp / refractVector.y) + samplePos;

        float result = distance(worldSpace, samplePos);

        result = pow(result / depth * 0.5, 2.0);

        albedo.rgb = vec3(result);
    }


    outAlbedo      = albedo;
    outPassthrough = vec4(normals * 0.5 + 0.5, 1.0);
}