attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;

out vec3 color;
out vec3 worldSpace;
out vec2 texcoord;

flat out vec3 normal;
flat out int materialIDs;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView, shadowModelViewInverse;
uniform vec3 cameraPosition;
uniform vec3 shadowLightPosition;
uniform vec3 sunPosition;
uniform float viewWidth, viewHeight;
uniform float frameTimeCounter;
uniform int frameCounter;


#include "/lib/Utility/Mat_IDs.glsl"
#include "/lib/Utility/Space_Transformations.glsl" 

#include "/lib/Vertex/Shadow_Transform.glsl"
#include "/lib/Vertex/Vertex_Displacement.glsl"

#include "/lib/Fragment/Shadow_Distortion.glsl"


void projectShadowViewSpace(out vec4 gl_Position, vec3 worldSpace) {
	gl_Position = vec4(projMAD(shadowProjection, transMAD(calculateShadowViewMatrix(), worldSpace)), worldSpace.z * shadowProjection[2].w + shadowProjection[3].w);
    gl_Position.xyz = gl_Position.xyz / vec3(vec2(distortionFactor(gl_Position.xy)), Shadow_Z_Stretch);
}

void calculateMatIDs(out int materialIDs) {
    int IDs  = 0;
    float id = mc_Entity.x;

    // Water
    if (id == WATER_FLOWING || id == WATER_STILL) IDs = 1;

    // Translucent
    if (id == LEAVES || id == VINES || id == TALLGRASS || id == DANDELION || id == ROSE ||
        id == WHEAT || id == LILYPAD || id == LEAVES2 || id == NEWFLOWERS || id == NETHER_WART ||
        id == DEAD_BUSH || id == CARROT || id == POTATO || id == COBWEB || id == SUGAR_CANE ||
        id == BROWN_SHROOM || id == RED_SHROOM) IDs = 2;

    // Emitters
    if (id == TORCH || id == FIRE || id == LAVAFLOWING || id == LAVASTILL || id == GLOWSTONE ||
        id == SEA_LANTERN || id == LAMP_ON || id == BEACON || id == END_ROD) IDs = 3;

    materialIDs = IDs;
}


void main() {
    color      = gl_Color.rgb;
    texcoord   = gl_MultiTexCoord0.xy;
    worldSpace = transMAD(shadowModelViewInverse, transMAD(gl_ModelViewMatrix, gl_Vertex.xyz)) + cameraPosition;
    normal     = normalize(gl_NormalMatrix * gl_Normal);

    calculateMatIDs(materialIDs);
    calculateVertexDisplacement(worldSpace, vec2(1.0), materialIDs);
    
    projectShadowViewSpace(gl_Position, worldSpace - cameraPosition);
}