attribute vec4 mc_Entity;
attribute vec4 at_tangent;
attribute vec4 mc_midTexCoord;

out vec4 color;
out vec3 worldSpace;
out vec3 viewSpace;
out vec3 tangentViewSpace;
out vec2 texcoord;
out vec2 vertexLightmaps;

flat out mat3 tbnMatrixWorld;
flat out mat3 tbnMatrixView;
flat out mat3x2 atlasTileInfo;
flat out vec2 midCoord;
flat out vec2 tileSize;
flat out float parallaxDepth;
flat out int materialIDs;

uniform sampler2D texture;
uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;
uniform vec3 cameraPosition;
uniform float viewWidth, viewHeight;
uniform float frameTimeCounter;
uniform int frameCounter;

#define atlasTileOffset     atlasTileInfo[0]
#define atlasTileSize       atlasTileInfo[1]
#define atlasTileResolution atlasTileInfo[2]


#include "/lib/Utility/Mat_IDs.glsl"
#include "/lib/Utility/Space_Transformations.glsl"

#include "/lib/Vertex/Vertex_Displacement.glsl"


void projectViewSpace(out vec4 gl_Position, vec3 worldSpace) {
    vec3 viewSpace = mat3(gbufferModelView) * (worldSpace - cameraPosition);
	gl_Position    = vec4(projMAD(gbufferProjection, viewSpace), viewSpace.z * gbufferProjection[2].w);
    gl_Position.xy = temporalJitter() * gl_Position.w + gl_Position.xy;
}

void calculateVertexLightmaps(out vec2 vertexLightmaps) {
    vertexLightmaps = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
}

void calculateTBN(out mat3 viewTBN, out mat3 worldTBN) {
	vec3 normal   = gl_NormalMatrix * gl_Normal;
	vec3 tangent  = gl_NormalMatrix * (at_tangent.xyz / at_tangent.w);
         viewTBN  = mat3(tangent, cross(tangent, normal), normal);
         worldTBN = mat3(gbufferModelViewInverse) * viewTBN;
}

void calculateTangentViewSpace(in vec3 viewSpace, in mat3 viewTBN, out vec3 tangentViewSpace) {
    tangentViewSpace = viewSpace * viewTBN;
}

void calculateAtlasTileInfo(inout mat3x2 atlasTileInfo) {
    ivec2 textureResolution = textureSize2D(texture, 0);

    atlasTileSize       = abs(gl_MultiTexCoord0.st - mc_midTexCoord.xy);
    atlasTileOffset     = mc_midTexCoord.xy - atlasTileSize;
    atlasTileSize      *= 2.0;

    atlasTileResolution = round(atlasTileSize * textureResolution);
    atlasTileSize       = atlasTileResolution / textureResolution;
    atlasTileOffset     = round(atlasTileOffset * textureResolution) / textureResolution;
}

void calculateMatIDs(out int materialIDs) {
    int IDs  = 0;
    float id = mc_Entity.x;

    // Water
    if (id == WATER_FLOWING || id == WATER_STILL) IDs = 1;

    // Translucent
    if (id == LEAVES || id == VINES || id == TALLGRASS || id == DANDELION || id == ROSE ||
        id == WHEAT || id == LILYPAD || id == LEAVES2 || id == NEWFLOWERS || id == NETHER_WART ||
        id == DEAD_BUSH || id == CARROT || id == POTATO || id == COBWEB || id == SUGAR_CANE ||
        id == BROWN_SHROOM || id == RED_SHROOM) IDs = 2;

    // Emitters
    if (id == TORCH || id == FIRE || id == LAVAFLOWING || id == LAVASTILL || id == GLOWSTONE ||
        id == SEA_LANTERN || id == LAMP_ON || id == BEACON || id == END_ROD) IDs = 3;

    materialIDs = IDs;
}


void main() {
    color      = gl_Color;
    texcoord   = mat2(gl_TextureMatrix[0]) * gl_MultiTexCoord0.st + gl_TextureMatrix[0][3].xy;
    viewSpace  = transMAD(gl_ModelViewMatrix, gl_Vertex.xyz);
    worldSpace = mat3(gbufferModelViewInverse) * viewSpace + cameraPosition;

    calculateVertexLightmaps(vertexLightmaps);
    calculateMatIDs(materialIDs);
    calculateTBN(tbnMatrixView, tbnMatrixWorld);
    calculateTangentViewSpace(viewSpace, tbnMatrixView, tangentViewSpace);
    calculateAtlasTileInfo(atlasTileInfo);
    calculateVertexDisplacement(worldSpace, vertexLightmaps, materialIDs);

    projectViewSpace(gl_Position, worldSpace);
}